/**
 * Exposes some utilities to work with unix time.
 *
 * See [Time](https://package.elm-lang.org/packages/elm/time/latest/Time) Elm package.
 *
 * @since 0.5.0
 */
import { interval } from 'rxjs';
import { map } from 'rxjs/operators';
/**
 * Get the current unix time as a `Task`.
 * @category constructors
 * @since 0.5.0
 */
export function now() {
    return () => Promise.resolve(new Date().getTime());
}
/**
 * Get the current unix time periodically.
 * @category utils
 * @since 0.5.0
 */
export function every(time, f) {
    return interval(time).pipe(map(() => f(new Date().getTime())));
}
