/**
 * Integration with _Redux DevTool Extension_.
 *
 * Please check the [docs](https://github.com/zalmoxisus/redux-devtools-extension/tree/master/docs/API) fur further information.
 *
 * @since 0.5.0
 */
import * as IO_ from 'fp-ts/lib/IO';
import * as O from 'fp-ts/lib/Option';
import { Dispatch } from '../Platform';
import { Debugger, Global } from './commons';
import Option = O.Option;
import IO = IO_.IO;
declare type Unsubscription = () => void;
/**
 * Defines a _Redux DevTool Extension_ object.
 * @category model
 * @since 0.5.0
 */
export interface Extension {
    connect: <Model, Msg>() => Connection<Model, Msg>;
}
/**
 * Defines a _Redux DevTool Extension_ connection object.
 * @category model
 * @since 0.5.0
 */
export interface Connection<Model, Msg> {
    subscribe: (listener?: Dispatch<DevToolMsg>) => Unsubscription;
    send(action: null, state: LiftedState<Model>): void;
    send(action: Msg, state: Model): void;
    init: (state: Model) => void;
    error: (message: unknown) => void;
    unsubscribe: () => void;
}
declare type DevToolMsg = Start | Action | Monitor;
interface Start {
    type: 'START';
}
interface Action {
    type: 'ACTION';
    payload: unknown;
}
interface Monitor {
    type: 'DISPATCH';
    payload: {
        type: 'JUMP_TO_STATE' | 'JUMP_TO_ACTION' | 'RESET' | 'ROLLBACK' | 'COMMIT' | 'IMPORT_STATE' | 'TOGGLE_ACTION';
        [k: string]: unknown;
    };
    [k: string]: unknown;
}
interface LiftedState<Model> {
    actionsById: Record<string, unknown>;
    computedStates: Array<{
        state: Model;
    }>;
    currentStateIndex: number;
    nextActionId: number;
    skippedActionIds: number[];
    stagedActionIds: number[];
    isPaused: boolean;
}
/**
 * Gets a _Redux DevTool Extension_ connection in case the extension is available
 * @category utils
 * @since 0.5.0
 */
export declare function getConnection<Model, Msg>(global: Global): IO<Option<Connection<Model, Msg>>>;
/**
 * **[UNSAFE]** Debug through _Redux DevTool Extension_
 * @category constructors
 * @since 0.5.4
 */
export declare function reduxDevToolDebugger<Model, Msg>(connection: Connection<Model, Msg>): Debugger<Model, Msg>;
export {};
