"use strict";
/**
 * A specialization of `Program` with the capability of mapping `Model` to `View`
 * and rendering it into a DOM node.
 *
 * `Html` is a base abstraction in order to work with any library that renders html.
 *
 * @since 0.5.0
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var operators_1 = require("rxjs/operators");
var platform = __importStar(require("./Platform"));
var Sub_1 = require("./Sub");
/**
 * Maps a view which carries a message of type `A` into a view which carries a message of type `B`.
 * @category Functor
 * @since 0.5.0
 */
function map(f) {
    return function (ha) { return function (dispatch) { return ha(function (a) { return dispatch(f(a)); }); }; };
}
exports.map = map;
/**
 * Returns a `Program` specialized for `Html`.
 *
 * It needs a `view()` function that maps `Model` to `Html`.
 *
 * Underneath it uses `Platform.program()`.
 * @category constructors
 * @since 0.5.0
 */
function program(init, update, view, subscriptions) {
    if (subscriptions === void 0) { subscriptions = function () { return Sub_1.none; }; }
    var _a = platform.program(init, update, subscriptions), dispatch = _a.dispatch, cmd$ = _a.cmd$, sub$ = _a.sub$, model$ = _a.model$;
    var html$ = model$.pipe(operators_1.map(view));
    return { dispatch: dispatch, cmd$: cmd$, sub$: sub$, model$: model$, html$: html$ };
}
exports.program = program;
/**
 * Same as `program()` but with `Flags` that can be passed when the `Program` is created in order to manage initial values.
 * @category constructors
 * @since 0.5.0
 */
function programWithFlags(init, update, view, subscriptions) {
    return function (flags) { return program(init(flags), update, view, subscriptions); };
}
exports.programWithFlags = programWithFlags;
/**
 * Stops the `program` when `signal` Observable emits a value.
 * @category combinators
 * @since 0.5.4
 */
function withStop(signal) {
    return function (program) {
        var platformProgram = platform.withStop(signal)(program);
        return __assign(__assign({}, platformProgram), { html$: program.html$.pipe(operators_1.takeUntil(signal)) });
    };
}
exports.withStop = withStop;
/**
 * Runs the `Program`.
 *
 * Underneath it uses `Platform.run()`.
 *
 * It subscribes to the views stream (`html$`) and runs `Renderer` for each new value.
 * @category utils
 * @since 0.5.0
 */
function run(program, renderer) {
    var dispatch = program.dispatch, html$ = program.html$;
    html$.subscribe(function (html) { return renderer(html(dispatch)); });
    return platform.run(program);
}
exports.run = run;
