"use strict";
/**
 * Makes http calls to remote resources as `Cmd`s.
 *
 * See [Http](https://package.elm-lang.org/packages/elm/http/latest/Http) Elm package.
 *
 * @since 0.5.0
 */
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var Arr = __importStar(require("fp-ts/lib/Array"));
var E = __importStar(require("fp-ts/lib/Either"));
var O = __importStar(require("fp-ts/lib/Option"));
var Rec = __importStar(require("fp-ts/lib/Record"));
var Semigroup_1 = require("fp-ts/lib/Semigroup");
var T = __importStar(require("fp-ts/lib/Task"));
var function_1 = require("fp-ts/lib/function");
var pipeable_1 = require("fp-ts/lib/pipeable");
var rxjs_1 = require("rxjs");
var ajax_1 = require("rxjs/ajax");
var operators_1 = require("rxjs/operators");
/**
 * @category destructors
 * @since 0.5.0
 */
function toTask(req) {
    return function () { return xhrOnlyBody(req).toPromise(); };
}
exports.toTask = toTask;
/**
 * Executes as `Cmd` the provided call to remote resource, mapping the result to a `Msg`.
 *
 * Derived from [`sendBy`](#sendBy).
 *
 * @since 0.5.0
 */
function send(f) {
    return sendBy(function_1.flow(E.map(function (r) { return r.body; }), f));
}
exports.send = send;
/**
 * Executes as `Cmd` the provided call to remote resource, mapping the full Response to a `Msg`.
 *
 * @since 0.6.0
 */
function sendBy(f) {
    return function_1.flow(xhr, toMsg(f));
}
exports.sendBy = sendBy;
/**
 * @category constructors
 * @since 0.5.0
 */
function get(url, decoder) {
    return {
        method: 'GET',
        headers: {},
        url: url,
        body: undefined,
        expect: decoder,
        timeout: O.none,
        withCredentials: false
    };
}
exports.get = get;
/**
 * @category constructors
 * @since 0.5.0
 */
function post(url, body, decoder) {
    return {
        method: 'POST',
        headers: {},
        url: url,
        body: body,
        expect: decoder,
        timeout: O.none,
        withCredentials: false
    };
}
exports.post = post;
var fromStrArr = Rec.fromFoldableMap(Semigroup_1.getLastSemigroup(), Arr.array);
var xhrOnlyBody = function_1.flow(xhr, extractBody());
function toMsg(project) {
    return operators_1.map(function_1.flow(project, O.some, T.of));
}
function extractBody() {
    return operators_1.map(E.map(function (response) { return response.body; }));
}
function xhr(req) {
    return pipeable_1.pipe(toXHRRequest(req), ajax_1.ajax, operators_1.map(function_1.flow(toResponse, decodeWith(req.expect))), operators_1.catchError(function (e) { return rxjs_1.of(E.left(toHttpError(req, e))); }));
}
function toXHRRequest(req) {
    return __assign(__assign({}, req), { timeout: O.getOrElse(function () { return 0; })(req.timeout), async: true, responseType: 'text' });
}
function toResponse(resp) {
    return {
        url: resp.request.url,
        status: { code: resp.status, message: resp.xhr.statusText },
        headers: toResponseHeaders(resp.xhr),
        body: typeof resp.response === 'string' && resp.response.length > 0 ? resp.response : '{}'
    };
}
function decodeWith(decoder) {
    return function (response) {
        return pipeable_1.pipe(
        // By spec parsing json can only throw `SyntaxError`
        E.parseJSON(response.body, function (e) { return e.message; }), E.chain(decoder), E.bimap(function (value) { return ({ _tag: 'BadPayload', value: value, response: response }); }, function (body) { return (__assign(__assign({}, response), { body: body })); }));
    };
}
function toHttpError(req, e) {
    if (e instanceof ajax_1.AjaxTimeoutError) {
        return { _tag: 'Timeout' };
    }
    if (e instanceof ajax_1.AjaxError && e.status === 404) {
        return { _tag: 'BadUrl', value: req.url };
    }
    if (e instanceof ajax_1.AjaxError && e.status !== 0) {
        return {
            _tag: 'BadStatus',
            response: {
                url: e.request.url,
                status: { code: e.status, message: e.xhr.statusText },
                headers: toResponseHeaders(e.xhr),
                body: e.response
            }
        };
    }
    return { _tag: 'NetworkError', value: e instanceof Error ? e.message : '' };
}
function toResponseHeaders(xhr) {
    return pipeable_1.pipe(xhr.getAllResponseHeaders(), O.fromPredicate(function (x) { return x.length > 0; }), O.map(function (hs) { return hs.trim().split(/[\r\n]+/); }), O.map(Arr.map(function (line) { return line.split(': '); })), O.map(function (kvs) { return fromStrArr(kvs, function (_a) {
        var k = _a[0], v = _a[1];
        return [k, v];
    }); }), O.getOrElse(function () { return ({}); }));
}
